@@ -10,8 +10,9 @@ __author__ = 'Pavel Zhukov (gelios@gmail.com)'
 
 import urllib
 import urllib2
-from cookielib import CookieJar
+from cookielib import CookieJar, Cookie
 import logging
+from copy import copy
 
 BASIC_HEADERS = {
     'User-Agent':    'Mozilla/5.0 (Windows; U; Windows NT 5.1; ru; rv:1.9.0.7) Gecko/2009021910 Firefox/3.0.7 GTB5',
@@ -25,13 +26,29 @@ RHEADERS = [BASIC_HEADERS]
 class ImpropertlyConfigured(Exception):
     pass
 
+
+def force_unicode(values, encoding):
+#Convert values to unicode if it's not unicode
+    if hasattr(values, 'items'):
+        #This is a dict
+        values = [(k, v) for k, v in values.items()]
+    encoded_values = []
+    for k, v in values:
+        if isinstance(k, unicode):
+            k = k.encode(encoding)
+        if isinstance(v, unicode):
+            v = v.encode(encoding)
+        encoded_values.append((k, v))
+    return encoded_values
+
 class Bot(object):
     '''
     Basic HTTP crawler builded against urllib/urllib2
-    This is just a current state storage (cookie, referral that can be used for crawling
+    This is just a current state storage (cookie and referral that can be used for crawling)
     '''
 
-    def __init__(self, send_referral=True, parse_cookies=True, proxies=None, cookiejar=None, add_headers=None):
+    def __init__(self, send_referral=True, parse_cookies=True, proxies=None, cookiejar=None, add_headers=None,
+                 encoding='utf-8'):
         '''
         Set basic headers settings
         '''
@@ -42,8 +59,9 @@ class Bot(object):
         if cookiejar is None:
             self.cj = CookieJar()
         self.browser = None
-        self._add_headers = {}
+        self._add_headers = add_headers or {}
         self.current_url = None
+        self.encoding = encoding
 
     def refresh_connector(self):
         '''
@@ -71,9 +89,10 @@ class Bot(object):
         Open target url
         Drop previous refferer
         Save cookies
+        Encode get and post values to self.encoding
         '''
         if self.browser is None:
-            self.refresh_connector()
+            self._create_connector()
         target = url
         #Remove referrer if exists
         if referrer:
@@ -82,12 +101,14 @@ class Bot(object):
             del self.headers['Referrer']
         self.browser.add_headers = self.headers
         if get:
+            get = force_unicode(get, self.encoding)
             target = url + '?' + urllib.urlencode(get)
         logging.debug('url: %s' % target)
         logging.debug('headers %s' % self.headers)
         logging.debug('post: %s' % post)
         logging.debug('referrer: %s' % referrer)
         if post:
+            post = force_unicode(post, self.encoding)
             self.response = self.browser.open(target, urllib.urlencode(post))
         else:
             self.response = self.browser.open(target)
@@ -95,7 +116,7 @@ class Bot(object):
         return self.response
 
     def follow(self, url, post=None, get=None):
-        self.response = self.open(url, post=post, get=get, referrer=self.current_url)
+        self.open(url, post=post, get=get, referrer=self.current_url)
         return self.response
 
     def open_http(self, method, url, values):
@@ -103,9 +124,39 @@ class Bot(object):
         Specically for lxml submit_form
         '''
         if method == 'GET':
-            self.follow(url, get=values)
+            return self.follow(url, get=values)
         elif method == 'POST':
-            self.follow(url, post=values)
+            return self.follow(url, post=values)
         else:
             raise ValueError('method parametr in bot.open_http should be eiser GET or POST')
 
+    def add_cookie(self, **kwargs):
+        '''
+        Add a Cookie to CookieJar used by OpenerDirector
+        You can specify any Cookie params, not-specified params will be setted by default
+        '''
+        default_cookie = {
+            'version': 0,
+            'name': 'name',
+            'value': '1',
+            'port': None,
+            'port_specified': False,
+            'domain': 'www.example.com',
+            'domain_specified': False,
+            'domain_initial_dot': False,
+            'path': '/',
+            'path_specified': True,
+            'secure': False,
+            'expires': None,
+            'discard': True,
+            'comment': None,
+            'comment_url': None,
+            'rest': {'HttpOnly': None},
+            'rfc2109': False
+        }
+        default_cookie.update(kwargs)
+        self.cj.set_cookie(Cookie(**default_cookie))
+
+    def clone(self):
+        ''' Shortcut for copy '''
+        return copy(self)
